<?php

namespace SiteZen\Telemetry\Bootstrap;

use SiteZen\Telemetry\Connectors\Database;
use SiteZen\Telemetry\Connectors\Server;
use SiteZen\Telemetry\Connectors\Application;

class Process
{
    var $data;
    var $db;

    public function __construct(array $config)
    {
        $payload = file_get_contents("php://input");
        $request = json_decode($payload, true);

        if (!is_array($request) || !isset($request['metric'])) {
            $this->data = ["error" => "Unknown request"];
            return $this;
        }

        $db = new \SiteZen\Telemetry\Connectors\Database($config, $request['parameters']??[]);
        $this->db = $db;

        switch ($request['metric']) {
            case 'server:cpu':
                $data = Server::cpu();
                break;
            case 'server:memory':
                $data = Server::memory();
                break;
            case 'server:storage':
                $data = Server::storage();
                break;
            case 'server:system':
                $data = Server::system();
                break;
            case 'server:all':
                $data = [
                    'cpu' => Server::cpu(),
                    'memory' => Server::memory(),
                    'storage' => Server::storage(),
                    'system' => Server::system(),
                    'php' => Server::php(),
                ];
                break;
            case 'database:usage':
                $data = $this->db->usage();
                break;
            case 'database:largestTables':
                $data = $this->db->largestTables();
                break;

            case 'database:connections':
                $data = $this->db->connections();
                break;

            case 'database:insight':
                $data = $this->db->insight();
                break;
            case 'database:analyse':
                $data = $this->db->analyzeProcessList();
                break;
            case 'database:processlist':
                $data = $this->db->processList();
                break;
            case 'database:all':
                $data = [
                    'usage' => $this->db->usage(),
                    'largestTables' => $this->db->largestTables(),
                    'connections' => $this->db->connections(),
                    'insight' => $this->db->insight(),
                    'analyse' => $this->db->analyzeProcessList(),
                ];
                break;


            case 'application:all':
                $data = [
                    'platform' => $config['platform']??'unknown',
//                    'version' => Application::version(),
                ];
                break;

            default:
                $data = ["error" => "Unknown metric request"];
                break;

        }
        $this->data = $data;
        $this->db->close();
        return $this;
    }

    public function toArray(): array
    {
        return $this->data;
    }
}
